/******************************************************************************
//
// FILE:		candrv_public.h 
//
// DESCRIPTION: Constants, structures, and function declarations shared with
//				this driver and the dll.
//
// DOCUMENTS:	-
//
******************************************************************************/

/******************************************************************************
// NESTING HEADER
******************************************************************************/
#ifndef CANDRV_PUBLIC_H
#define CANDRV_PUBLIC_H

/******************************************************************************
// INCLUDE FILES
******************************************************************************/

/******************************************************************************
// MANIFEST CONSTANTS
******************************************************************************/

//
// Various constants
//
#define	CAN_MAX_MSG_LENGTH		8
#define	CAN_MSG_RTR				0x80000000
#define	CAN_MSG_EXTENDED		0x40000000

//
// IOCTL codes.
//

//
// Returns information about the characteristics of the attached communication
// device.
// Format of data is defined by type CanDrvProperties.
//
#define IOCTL_CAN_GET_PROPERTIES	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the current baud-rate setting.
// Format of data is defined by type CanDrvBaudrate.
//
#define IOCTL_CAN_GET_BAUDRATE		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x802, METHOD_BUFFERED, FILE_ANY_ACCESS)
//
// Sets the speed at which characters are sent over the CAN net.
// The caller must have Super User privilege to perform this operation,
// i. e. CanOpenEx must have been used to open the interface
// Format of data is defined by type CanDrvBaudrate.
//
#define IOCTL_CAN_SET_BAUDRATE		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x803, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Resizes the driver's internal type-ahead receive fifo.
// The caller must have Super User privilege to perform this operation,
// i. e. CanOpenEx must have been used to open the interface
// Format of data is defined by type CanDrvQueueSize.
//
#define IOCTL_CAN_SET_QUEUE_SIZE	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the current settings for time-outs on the device.
// Format of data is defined by type CanDrvTimeouts.
//
#define IOCTL_CAN_GET_TIMEOUTS		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Sets up time-out constraints for device I/O operations.
// The caller must have Super User privilege to perform this operation,
// i. e. CanOpenEx must have been used to open the interface
// Format of data is defined by type CanDrvTimeouts.
//
#define IOCTL_CAN_SET_TIMEOUTS		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x806, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Sets the device to a known state and cancels all pending requests.
// The caller must have Super User privilege to perform this operation,
// i. e. CanOpenEx must have been used to open the interface
// No data is supplied.
//
#define IOCTL_CAN_RESET_DEVICE		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x807, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the current values of the performance statistics for the device.
// Format of data is defined by type CanDrvStatistics.
//
#define IOCTL_CAN_GET_STATS			\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Resets the performance statistics for the device.
// No data is supplied.
//
#define IOCTL_CAN_CLEAR_STATS		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Reserved for CanOpenEx
// No data is supplied.
//
#define IOCTL_CAN_OPEN_EX			\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80a, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Reads the max frequency by which the CAN controller registers can be
// accessed.
// Format of data is defined by type ULONG.
//
#define IOCTL_CAN_GET_REG_ACCESS_FREQ	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80b, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Reads the contents of CAN controller registers or
// Sets CAN controller registers.
// Format of data is defined by type CanDrvAccessRegister.
//
#define IOCTL_CAN_ACCESS_REG		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80c, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Starts generation of can messages.
// Via this function the can driver can act as a test generator.
// Format of data is defined by type CanDrvGenMsgStart.
//
#define IOCTL_CAN_GEN_MSG_START		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80d, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Stop generation of can messages.
// No data is supplied.
//
#define IOCTL_CAN_GEN_MSG_STOP		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80e, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the trace mask.
// Format of data is defined by type CanDrvTraceMask.
//
#define IOCTL_CAN_GET_TRACE_MASK	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x80f, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Sets the trace mask.
// Format of data is defined by type CanDrvTraceMask.
//
#define IOCTL_CAN_SET_TRACE_MASK	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x810, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the size of the trace buffer.
// Format of data is defined by type CanDrvTraceSize.
//
#define IOCTL_CAN_GET_TRACE_SIZE	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x811, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Resizes the trace buffer.
// Format of data is defined by type CanDrvTraceSize.
//
#define IOCTL_CAN_SET_TRACE_SIZE	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x812, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Resets the trace buffer.
// No data is supplied.
//
#define IOCTL_CAN_CLEAR_TRACE		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x813, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the first trace line.
// Format of data is defined by type CanDrvGetTraceLine.
//
#define IOCTL_CAN_GET_FIRST_TRACE_LINE	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x814, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the next trace line.
// Format of data is defined by type CanDrvGetTraceLine.
//
#define IOCTL_CAN_GET_NEXT_TRACE_LINE	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x815, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns all trace lines.
// Format of data is defined by type CanDrvGetTrace.
//
#define IOCTL_CAN_GET_TRACE			\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x816, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns information about the size of the CanDrvProperties structure which
// is returned as a result of IOCTL_CAN_GET_PROPERTIES.
// Format of data is defined by type ULONG.
//
#define IOCTL_CAN_GET_PROPERTIES_SIZE	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x817, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the properties of the CAN to keyboard translation.
// Format of data is defined by type CanDrvKeyboardProperties.
//
#define IOCTL_CAN_GET_KEYBOARD_PROP	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x818, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Sets the properties of the CAN to keyboard translation.
// Format of data is defined by type CanDrvKeyboardProperties.
//
#define IOCTL_CAN_SET_KEYBOARD_PROP	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x819, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the properties of the CAN to mouse translation.
// Format of data is defined by type CanDrvMouseProperties.
//
#define IOCTL_CAN_GET_MOUSE_PROP	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81a, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Sets the properties of the CAN to mouse translation.
// Format of data is defined by type CanDrvMouseProperties.
//
#define IOCTL_CAN_SET_MOUSE_PROP	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81b, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Returns the current timestamp.
// Format of data is defined by type CanTimeStamp.
//
#define IOCTL_CAN_GET_TIMESTAMP		\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81c, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Reserved for CanSend.
// Format of input data is defined by type CanMsg.
// Format of output data is defined by type CanTimeStamp.
//
#define IOCTL_CAN_SEND				\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81d, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Obsolete ,replaced by IOCTL_CAN_ENABLE_REMREPLY
//
#define IOCTL_CAN_ADD_REMREPLY	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81e, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Obsolete ,replaced by IOCTL_CAN_DISABLE_REMREPLY
//
#define IOCTL_CAN_REMOVE_REMREPLY	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x81f, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Requests the driver to enable a remote frame reply.
// Format of data is defined by type CanMsgId.
//
#define IOCTL_CAN_ENABLE_REMREPLY	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x820, METHOD_BUFFERED, FILE_ANY_ACCESS)

//
// Requests the driver to disable a remote frame reply.
// Format of data is defined by type CanMsgId.
//
#define IOCTL_CAN_DISABLE_REMREPLY	\
	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x821, METHOD_BUFFERED, FILE_ANY_ACCESS)

/******************************************************************************
// TYPES
******************************************************************************/

//
// Definition of a CAN message.
//
typedef ULONG CanMsgId;

typedef struct _CanMsg
{
	CanMsgId	id;							// Id of can message.	
	UCHAR		data[CAN_MAX_MSG_LENGTH];	// Data of Can message.
} CanMsg;

//
// Definition of a CAN frame types.
//
typedef ULONG CanFrameType;

#define CAN_FRAME_STANDARD 0x1
#define CAN_FRAME_EXTENDED 0x2
#define CAN_FRAME_REMOTE   0x4   
// CAN_FRAME_REMOTE_ONLY can be used with CanReceiveEx to receive only remote frames.
#define CAN_FRAME_REMOTE_ONLY 0x8

//
// Software fifo field used for storing received CAN messages.
//
typedef struct _CanTimeStamp
{
	ULONG low;
	ULONG high;
} CanTimeStamp;

typedef struct _RxFifoMsg
{
	CanTimeStamp	timeStamp;	// Timestamp in 100-nano seconds.
	ULONG			length;		// Number of Data bytes, i e "sizeof(Can.Data)".
	CanMsg			can;		// Can message.
} RxFifoMsg;

//
// IOCTL data structures.
//

typedef struct _CanDrvVersion
{
    ULONG major;	// Major version number of loaded driver, updated
					// when the interface is changed.
    ULONG minor;	// Minor version number of loaded driver.
} CanDrvVersion;

typedef struct _CanDrvProperties
{
    CanDrvVersion	version;				// Version number.
	CanFrameType	frameTypes;				// Supported frametypes.
	ULONG			offsetRegKeySubPath;	// offset to string in
											// strings-array that identifies
											// the registry key of the driver
											// relative to "HKEY_LOCAL_MACHINE\
											// SYSTEM\CurrentControlSet\
											// Services\".
	ULONG			offsetController;		// offset to string in
											// strings-array that identifies
											// the controller.
	WCHAR			strings[1];				// Contains multiple strings.
} CanDrvProperties;

typedef ULONG CanDrvBaudrate;	// Baudrate (bits/s)

typedef struct _CanDrvQueueSize
{
	ULONG rxFifoSize;	// Max. number of messages that the type-ahead
						// receive fifo can contain.
} CanDrvQueueSize;

typedef struct _CanDrvTimeouts
{
	ULONG	txFailureTimeout;	// Indicates how many milliseconds a
								// transmission can be pending before
								// it should be considered to have failed.
	ULONG	busCheckPeriod;		// Indicates at what interval in milliseconds
								// the status of the CAN bus should be checked.
	ULONG	busOffRecoverDelay;	// Indicates the delay in milliseconds to wait
								// before an attempt should be made to recover
								// from a bus off state.
} CanDrvTimeouts;

typedef struct _CanDrvStatistics
{
	ULONG	rxMsgCntr;		// The number CAN messages received so far.
	ULONG	rxDataCntr;		// The number of data bytes received so far.
	ULONG	txMsgCntr;		// The number CAN messages transmitted so far.
	ULONG	txDataCntr;		// The number of data bytes transmitted so far.
	ULONG	hwOvrnCntr;		// The number of times that the CAN controller has
							// detected a receive overrun.
	ULONG	busWarnCntr;	// The number of times that the CAN controller has
							// experienced a bus warning condition.
	ULONG	busOffCntr;		// The number of times that the CAN controller has
							// entered the bus off state.
	ULONG	rxFifoOvrnCntr;	// The number of times that a overrun has occurred
							// in the type-ahead receive fifo. If this occurs
							// then the size of the type-ahead receive fifo
							// needs to be increased, see configuration value
							// "RxFifoSize".
	ULONG	rxFifoMax;		// The maximum number of messages that the
							// type-ahead receive fifo have contained.
} CanDrvStatistics;

typedef struct _CanDrvAccessRegister
{
	ULONG	count;			// The number of times all the entries in
							// "reg[]" should be repeated.
	ULONG	noRegs;			// The number of elements in reg array.
	struct {
		ULONG	count;		// The number of times access of this register
							// should be repeated.
		BOOLEAN	bWrite;		// Zero if the register should be read.
							// Non-zero if the register should be written. 
		ULONG	offset;		// The offset of the register.
		UCHAR	validBits;	// Indicates which bits in value that are valid.
							// A '1' means that bit is valid.
		UCHAR	value;		// The read value or the value to set .
	} reg[1];
} CanDrvAccessRegister;

typedef struct _CanDrvGenMsgStart
{
	ULONG	count;					// The number of times all the entries in
									// "entry[]" should be repeated.
									// 0xffffffff means infinite, i e
									// IOCTL_CAN_GEN_MSG_STOP is needed to stop
									// operation.
	ULONG	noEntries;				// The number of elements in entry array.
	struct
	{
		ULONG	count;				// The number of times this entry should be
									// repeated.
		struct
		{							// The number of 100-nano seconds that
			ULONG low;				// should elapse before an between all
			ULONG high;				// repetitions of this entry.
		} delay;

		BOOLEAN			bRtr;		// Should be set to TRUE if message is to
									// be sent as a remote frame.
		CanFrameType	frameType;	// should be set to:
									// CAN_FRAME_STANDARD if message is to be
									// sent as a standard frame.
									// CAN_FRAME_EXTENDED if message is to be
									// sent as an extended frame.
		ULONG			length;		// Number of Data bytes in canMsg.
		CanMsg			canMsg;		// The CAN message to send
	} entry[1];
} CanDrvGenMsgStart;

typedef ULONG CanDrvTraceMask;	// Trace mask.

typedef struct _CanDrvTraceSize
{
	ULONG	noLines;	// Trace buffer size in LineLength lines.
	ULONG	lineLength;	// Size in bytes of a trace line.
} CanDrvTraceSize;

typedef struct _CanDrvGetTraceLine
{
	struct {				// The next trace line should not be touched
		ULONG head;
		ULONG offset;
	} next;
	CHAR	line[1];		// The contents of a trace line.
							// Must be of CanDrvQueueSize.lineLength size.
} CanDrvGetTraceLine;

typedef struct _CanDrvGetTrace
{
	CHAR	lines[1];		// The contents of all trace lines separated
							// Each line occupies CanDrvQueueSize.lineLength.
							// Must be of CanDrvQueueSize.noLines * 
							// CanDrvQueueSize.lineLength size.
} CanDrvGetTrace;

typedef struct _CanDrvKeyboardProperties
{
	BOOLEAN	bEnabled;		// Zero if the CAN to keyboard translation is
							// or should be disabled otherwise Non-zero. 
} CanDrvKeyboardProperties;

//
// CAN to mouse move modes.
//
#define	CANTOMOUSE_MODE_REL 0x01	// Translation should be relative.
#define CANTOMOUSE_MODE_ABS	0x02	// Translation should be absolute.

typedef struct _CanDrvMouseProperties
{
	BOOLEAN	bEnabled;		// Zero if the CAN to mouse translation is or
							// should be disabled otherwise Non-zero.
	ULONG	mode;			// CAN to mouse current move mode.
	ULONG	availableModes;	// CAN to mouse available move modes.
} CanDrvMouseProperties;

/******************************************************************************
// FUNCTION PROTOTYPES 
******************************************************************************/

/******************************************************************************
// NESTING TRAILER 
******************************************************************************/
#endif
